import sys, os


def black(pyScript=sys.argv[0]):
    """Reformat python script which calls this function to look like PEP8
    Requires black library (pip install black)"""
    os.system('black "%s"' % pyScript)



import sys, os

def deleteThumb():
    os.remove(os.path.dirname(sys.argv[0]) + '\\thummb.db')


def cmd(*args):
    '''Input: string, string, string, ...
    Can run multiple commands to the windows terminal'''

    def cmdSystem(cmdList):
       msg = ''
       for cmd in cmdList:
           if msg == '':
               msg = msg + cmd
           else:
               msg = msg + ' & ' + cmd
       return msg

    cmdlist = list()
    for instruction in args:
        cmdlist.append(instruction)
    cmdlist = cmdSystem(cmdlist)
    os.system(cmdlist)



import os
from grtoolkit.Decorators import try_pass


def importPackage(package):
    """Programatically import library"""
    import importlib
    return importlib.import_module(package)


def packagePath(package):
    """Find local path of library"""
    my_module = importPackage(package)
    return my_module.__file__


def packageDir(package):
    return os.path.dirname(packagePath(package))


def runPipenv(path, *args, **kwargs):
    execCommand = f"pipenv run python {path}"
    for arg in args:
        execCommand+= f" {arg}"
    for kwarg in kwargs:
        execCommand+= f" {kwarg}"
    os.system(execCommand)

@try_pass
def dictionary_add_modify(dictionary, **kwargs2AddUpdate):
    """Add or modify kwargs to dictionary"""
    for k,v in kwargs2AddUpdate.items():
        dictionary.update({f"{k}":v})
    return dictionary



if __name__ == "__main__":
    pass



def pressureInFluid(f_perpendicular, A):
    """
    variables:
            f_perpendicular = normal force
            A = area"""
    p=f_perpendicular/A
    return p

def bulkStress(dp, dV, V0):
    """variables:
            dp=change in fluid pressure
            dV=change in subject volume
            V0=original subject volume"""
    # B = bulk stress / bulk strain
    B = dp/(dV/V0)
    return B



import os, sys
from grtoolkit.Decorators import try_pass
from grtoolkit.Windows import cmd
from grtoolkit.Storage import deleteDirectory

import requests, json, datetime
import grtoolkit, re

cwd = os.path.dirname(sys.argv[0])

def pypiJSON(package, version="0.0.0 Optinal"):
    '''Returns pypi JSON file using pypi API'''
    if version == "0.0.0 Optinal":
        r = requests.get(f'https://pypi.org/pypi/{package}/json')
    else:
        r = requests.get(f'https://pypi.org/pypi/{package}/{version}/json')
    return r.json()

def pypiVersion(package):
    '''Returns latest pypi version code from specified package name'''
    return pypiJSON(package)['info']['version']

def genDateVersion(package):
    '''Returns a new date version code after comparing with latest Pypi package version online
    Follows the format Year.Month.ReleaseNumber'''
    try:
        currVerSplit = pypiVersion(package).split(".")
    except:
        currVerSplit = "0.0.0".split(".")
        
    DATE = datetime.datetime.now()

    def prefix0str(interger):
        if len(str(interger)) == 1:
            return f"0{str(interger)}"
        else:
            return str(interger)

    # COMPARE IF THE YEAR AND MONTH MATCHES
    if DATE.strftime(f"%y.%m") == f"{currVerSplit[0]}.{prefix0str(currVerSplit[1])}":
        newVersion = DATE.strftime(f"%y.%m.{int(currVerSplit[2])+1}")
    else:
        newVersion = DATE.strftime(f"%y.%m.0")
    return newVersion

def updateSetupVer(setupfile=os.path.dirname(sys.argv[0])+"\\setup.py"):
    '''Updates setup.py in the specified directory to the next Date Version'''
    package_regex = re.compile('name=".*"')     
    version_regex = re.compile('version=".*"')

    setup = grtoolkit.Storage.File(setupfile)
    setup_txt = setup.read()
    package = package_regex.findall(setup_txt)[0].split('"')[1]     #Find package name from setup file
    currVersion = version_regex.findall(setup_txt)[0]               #Find version currently on setup file

    newsetup = grtoolkit.File.replaceWords(setup_txt,{currVersion:f'version="{genDateVersion(package)}"'})
    setup.write(newsetup)   #Overwrite setup.py

def Upload2Pypi():
    '''Recreates distribution & build, updates version on setup.py, pushes upload to Pypi
    Must be used in the same folder as setup.py'''
    if os.path.exists(cwd + "\\setup.py"):
        deleteDirectory(os.path.dirname(sys.argv[0])+'\\build\\')
        deleteDirectory(os.path.dirname(sys.argv[0])+'\\dist\\')
        deleteDirectory(os.path.dirname(sys.argv[0])+'\\__pycache__\\')

        updateSetupVer()

        if "PYPI_USER" in os.environ and "PYPI_PASS" in os.environ:
            cmd(f'cd {cwd}', 
                'python setup.py sdist bdist_wheel',
                f'twine upload dist/* -u {os.environ["PYPI_USER"]} -p {os.environ["PYPI_PASS"]}')
        else:
            cmd(f'cd {cwd}', 
                'python setup.py sdist bdist_wheel',
                f'twine upload dist/*')
    else:
        print(f"setup.py not found in {cwd}")

if __name__ == "__main__":
    Upload2Pypi()



from grtoolkit.Python import dictionary_add_modify
from grtoolkit.Math import solveEqs
from math import pi
import scipy.constants

def gravitationAttraction(find="F_g", printEq=False, **kwargs):
    """variables:
            F_g = force of attraction due to gravity
            G = 6.67e-11 N*m**2/kg**2
            m1, m2 = mass1, mass2
            r = distance between masses"""
    dictionary_add_modify(kwargs,G=6.67e-11)
    eq = list()
    eq.append("Eq(F_g,G*m1*m2/r**2)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def gravitationalPotentialEnergy(find="U", printEq=False, **kwargs):
    """variables:
            U = gravitational potential energy
            G = 6.67e-11 N*m**2/kg**2
            m1, m2 = mass1, mass2
            r = distance between masses"""
    dictionary_add_modify(kwargs,G=6.67e-11)
    eq = list()
    eq.append("Eq(U,-G*m1*m2/r)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def orbitVelocity(find="v_orb", printEq=False, **kwargs):
    """variables:
            v_orb = gravitational potential energy
            G = 6.67e-11 N*m**2/kg**2
            m_planet=mass of planet being orbited
            r=distance from center of planet being orbited"""
    dictionary_add_modify(kwargs,G=6.67e-11)
    eq = list()
    eq.append("Eq(v,(G*m_planet/r)**(1/2))")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def v_orb(find="v_orb", printEq=False, **kwargs):
        return orbitVelocity(find, printEq, **kwargs)

def orbitPeriod(r,v_orb):
        T= 2*pi*r/v_orb
        return T

def blackHoleMaxRadius(M):
        """If a nonrotating spherical mass distribution with total mass M has a radius less than its Schwarzschild radius Rs, it is called a black hole. The gravitational interaction prevents anything including light, from escaping from within a sphere with Radius Rs."""
        G=scipy.constants.G
        c=scipy.constants.c
        Rs=2*G*M/c**2
        return Rs



import pyodbc, os, sys
from grtoolkit.MSAccess import SQLQuery

PropertyTables = os.path.dirname(sys.argv[0]) + "\\PropertyTables.accdb"
PT = PropertyTables

def Farenheit(Celcius):
    return 9.0/5.0*Celcius+32.0

def Celcius(Farenheit):
    return (Farenheit-32.0)/9.0*5.0
    
def tblCode(shortcutName):
    return SQLQuery(PT, f"Select tblName from A00_VBA_Table_References where sqlFromVal='{shortcutName}'", fetch="val")

def SQLPT(SQL, fetch="all"):
    return SQLQuery(PT, SQL, fetch)


# sample = SQLPT(f"Select temp from {tblCode('water_t')} where temp>55")




import pickle, shutil, os, re
from grtoolkit.Decorators import try_pass
from grtoolkit.File import directoryLastValue

# class Pickle:
#     def __init__(self, fileName):
#         self.fileName = fileName

#     def save(self):
#         outfile = open(filename, 'wb')
#         pickle.dump(pickle_object, outfile)
#         outfile.close()

#     def load(self):
#         infile = open(filename, 'rb')
#         pickle_object = pickle.load(infile)
#         infile.close
#         return pickle_object

    

def savePickle(filename, pickle_object):
    outfile = open(filename, 'wb')
    pickle.dump(pickle_object, outfile)
    outfile.close()
    
def loadPickle(filename):
    infile = open(filename, 'rb')
    pickle_object = pickle.load(infile)
    infile.close
    return pickle_object

@try_pass
def deleteDirectory(path):
    shutil.rmtree(path)

class File:
    def __init__(self, fileName, enc=0):
        self.fileName = fileName
        self.enc_dict = {0:None, 1:"utf8", 2:"utf-16le"}
        try:
            self.encode = self.enc_options(enc)
        except:
            self.encode = enc

    def enc_options(self, interger):
        return self.enc_dict[interger]

    def write(self, content):
        """Overwrites to file - deletes what was there before
        If file did not exist it creates a file"""

        if self.encode != None:
            content = content.encode(self.encode)
            f = open(self.fileName, "wb")
        else:
            f = open(self.fileName, "w")
        f.write(content)
        f.close()

    def append(self, content, newline=True):
        """Appends content to existing file"""
        nextLine = "\n"
        if self.encode != None:
            nextLine = nextLine.encode(self.encode)
            content = content.encode(self.encode)
            f = open(self.fileName, "ab")
        else:
            f = open(self.fileName, "a")
        f.write(nextLine + content) if newline else f.write(content)
        f.close()

    def read(self):
        """Returns existing file content"""

        f = None
        t = None

        if self.encode != None:
            # t = f.read().decode(self.encode)
            f = open(self.fileName, "rb")
            t = f.read()
            f.close()

            # with open(self.fileName, "rb") as binary_file:
            #     # Read the whole file at once
            #     f = binary_file.read()

            # with open(self.fileName, "rb") as f:
            #     byte = f.read(1)
            #         while byte:
            #         # Do stuff with byte.
            #         byte = f.read(1)

        else:
            f = open(self.fileName)
            t = f.read()
            f.close()
        # f.close
        return t

    def print(self):
        """Prints existing file content"""

        print(self.read())

def search(rootFolder, viewPrint=False, rootInclude=False, depth=None, abs=True, lastValue=False):
    '''USAGE: 
folders, files = search(path, depth=None, abs=False)

PURPOSE
Application tool for os.walk

ARGUMENTS:
    viewPrint - print to console parent folder, subfolder, and files that search() is encountering
    rootInclude - include root search folder in the output folder search results
    depth - how many levels of folders should the program output, default is no limit
    abs - output absolute or relative paths. Default is absolute path.
    lastValue - true/false. return file name or absolute address'''

    files_recursive = list()
    folders_recursive = list()
    rootBaseDepth = rootFolder.count("\\")
    rootFolderLength = len(rootFolder)
    removeStartSlashes = lambda x: x[1:] if x[:1] =='\\' else x

    if rootInclude:
        folders_recursive = [f"{rootFolder}"] if abs else [f"{directoryLastValue(rootFolder)}"]

    for root, subfolders, files in os.walk(rootFolder):

        #CHECK/COMPARE DEPTHS
        depth_current = root.count("\\") - rootBaseDepth + 1
        if depth:
            if depth < depth_current:
                break #breaks out of for loop

        if viewPrint:
            print("Parent Directory:"); print(root)
            print("Subfolders:"); print(subfolders)
            print("Files"); print(files); print("\n")

        if abs: #absolute paths
            files_recursive = files_recursive + list(map(lambda x:f"{root}\\{x}",files))
            folders_recursive = folders_recursive + list(map(lambda x:f"{root}\\{x}",subfolders))
        else:   #relative paths
            files_recursive = files_recursive + list(map(lambda x:removeStartSlashes(root[rootFolderLength:] + '\\' + x) ,files))
            folders_recursive = folders_recursive + list(map(lambda x: removeStartSlashes(root[rootFolderLength:] + '\\' + x) ,subfolders))

        if lastValue:
            files_recursive = list(map(lambda x:directoryLastValue(x),files_recursive))
            folders_recursive = list(map(lambda x:directoryLastValue(x),folders_recursive))

    return folders_recursive, files_recursive

def regexList(unfilteredList, regex):
    '''Returns list filtered by regex'''
    item_regex = re.compile(regex, re.IGNORECASE)  # Regular Expression; dot star means find everything
    filteredList = list()
    for item in unfilteredList:
        filtering = item_regex.findall(item)
        if filtering:
            filteredList.append(item)
    return filteredList


import grtoolkit.Circuits.AC
import grtoolkit.Circuits.Filters
import grtoolkit.Circuits.OpAmps
import grtoolkit.Circuits.Transistors

from grtoolkit.Math import solveEqs

def BasicLaws(find, printEq=False, **kwargs):
    """variables: 
                V=voltage       (V)
                I=current       (A)
                R=resistance    (ohm) 
                P=power         (W)
                Q=charge        (C)
                G=conductance   (mho aka. siemens)"""
    eq = list()
    eq.append("Eq(V,I*R)")    
    eq.append("Eq(P,V*I)")
    eq.append("Eq(P,W/t)")    
    eq.append("Eq(V,W/Q)")
    eq.append("Eq(I,Q/t)")  
    eq.append("Eq(Q,integrate(I,(t,t0,t1)))")
    eq.append("Eq(G,1/R")
    eq.append("Eq(i,(v_higher-v_lower)/R)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def Resistance(find, printEq=False, **kwargs):
    """variables: 
                R=resistance
                p=density
                l=length
                A=cross sectional area"""
    eq = list()
    eq.append("Eq(R,p*l/A)")    
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def Capacitance(find="C", printEq=False, **kwargs):
    """
    Usage: the ratio of the charge on one plate of a capacitor to the voltage difference between the two plates, measured in farads (F).


    variables: 
                C = capacitance
                epsilon = permitivity of the dielectric material between plates
                A = surface area of each plate
                d = distance between plates"""
    eq = list()
    eq.append("Eq(C,epsilon*A/d)")    
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def KCL(i_list):
    """Sum of currents entering a node equal the sum of currents leaving a node"""
    return sum(i_list)

def KVL(v_list):
    """Sum of all voltages around a closed path (or loop) is 0"""
    return sum(v_list)

def ResistorsInSeries(r_list):
    """Resistors connected in series is the sum of the individual resistances"""
    return sum(r_list)

def ResistorsInParellel(r_list):
    sumOfInverse = sum([1/r for r in r_list])
    return 1/sumOfInverse

def voltageDivision(v_in, r_list_ordered, showWork=False):
    """voltage is divided among the resistors in direct proportion to their resistances; the larger the resistance, the larger the voltage drop."""
    r_total = sum(r_list_ordered)
    voltages = [r/r_total*v_in for r in r_list_ordered]
    print("Resistor ordered voltage division: ", voltages)
    return voltages

def currentDivision(i_in, r_branch_list_ordered, showWork=False):
    conductances = [Conductance(r) for r in r_branch_list_ordered]
    g_total = sum(conductances)
    currents = [g/g_total*i_in for g in conductances]
    if showWork:
        print("Branch ordered current division: ", currents) 
    return currents

def Conductance(r):
    return 1/r

def ConductanceInSeries(g_list):
    return ResistorsInParellel(g_list)

def ConductanceInParellel(g_list):
    return ResistorsInSeries(g_list)

def delta2wye(Ra, Rb, Rc):
    Rt = Ra+Rb+Rc
    R1 = Rb*Rc/Rt
    R2 = Rc*Ra/Rt
    R3 = Ra*Rb/Rt
    return R1, R2, R3

def wye2delta(R1, R2, R3):
    Rx = R1*R2 + R2*R3 + R3*R1
    Ra = Rx/R1
    Rb = Rx/R2
    Rc = Rx/R3
    return Ra, Rb, Rc

def NodalSimpleAnalysis(vh, vl, r):
    """Current flows from a higher potential to a lower potential in a resistor"""
    i = (vh-vl)/r
    return i

def NodalAnalysis(find="i", printEq=False, **kwargs):
    """variables: 
            i = current
            vh, vl = higher/lower voltage
            r = resistor
        usage:
            Current flows from a higher potential to a lower potential in a resistor"""
    eq = list()
    eq.append("Eq(i,(v_higher-v_lower)/r)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def supernode():
    print("""
        A supernode is formed by enclosing a (dependent or independent)
        voltage source connected between two nonreference nodes and any
        elements connected in parallel with it.""")

def meshAnalysis():
    # Revisit when simultaneous eq in grtoolkit.math complete
    print("""
        Sum of all voltages in a node""")

def sourceTransformation():
    # TO BE REVIEWED
    pass

def TheveninNorton():
    # TO BE REVIEWED
    pass

def maximumPowerTransfer():
    # TO BE REVIEWED
    pass


# python = "C:\Users\gabrielr\AppData\Local\Programs\Python\Python38-32\python.exe"
# pip = "C:\Users\gabrielr\AppData\Local\Programs\Python\Python38-32\python.exe\Scripts\pip.exe"

import sys, os
from pptx import Presentation
# from os.path import dirname
from grtoolkit.Storage import File
from grtoolkit.PowerPoint import duplicate_slide

# parent = dirname(sys.argv[0])
# parent = os.getcwd()
parent = r"D:\OneDrive\Development\Life\The Truth"
QUOTES = f'{parent}\\Quotes.txt'
OLD_PPT = f'{parent}\\QUOTE_PPT_TEMPLATE.pptm'
NEW_PPT = f'{parent}\\QUOTE_NEW.pptm'

# QUOTES = 

prs = Presentation(OLD_PPT)

QUOTESContent = File(QUOTES, enc=1).read()
QUOTESList = QUOTESContent.split("\n\n")

#DUPLICATE PPT SLIDE FOR EVERY 
# for i in range (1,len(QUOTESList)):
#     duplicate_slide(prs,0)

# i = 0
# for quote in QUOTESList:
#     if i>0:
#         duplicate_slide(prs,0)
#     i=i+1

#DUPLICATE SLIDE 0 FROM TEMPLATE
duplicate_slide(prs,0,len(QUOTESList)-1)
#CHANGE TEXT IN ALL SLIDES AS PER NOTEPAD FILE
i=0
for slide in prs.slides:
    for shape in slide.shapes:
        if hasattr(shape, "text"):
            shape.text_frame.clear()
            p=shape.text_frame.paragraphs[0]
            run = p.add_run()
            run.text = QUOTESList[i]
            font=run.font
            font.name='Gabriola'
    i=i+1
prs.save(NEW_PPT)


import math
from grtoolkit.Math import solveEqs, magnitude
# from grtoolkit.M import magnitude

################################ RADIAL
def v_rad(r,T):
    """
    Usage: velocity for uniform circular motion

    Variables: 
        T = period
        r = radius"""
    return 2*math.pi*r/T

def a_rad(printEq=False, **kwargs):
    """
    Usage: radial acceleration (direction is towards axis of rotation)

    Variables: 
        v_rad=radial velocity
        r = radius
        w = magnitude of angular velocity (w_z)"""
    find="a_rad"
    eq = list()
    eq.append("Eq(a_rad, v_rad**2/r)")
    eq.append("Eq(a_rad, w**2*r)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)


################################ TANGENTIAL

def v_tan(r, w_z):
    """
    Usage: velocity for uniform circular motion

    Variables: 
        r = radius
        w = magnitude of angular velocity (w_z)"""
    return r*magnitude(w_z)

def a_tan(find="a_tan", printEq=False, **kwargs):
    """
    Usage: radial acceleration (direction is towards axis of rotation)

    Variables: 
        a_tan=tangential acceleration
        dv=change in velocity
        dw=change in angular velocity
        dt=change in time
        r=radius
        alpha=rate of change of angular velocity
        """
    eq = list()
    eq.append("Eq(a_tan, dv/dt)")
    eq.append("Eq(a_tan, r*dw/dt)")
    eq.append("Eq(a_tan, r*alpha)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)


################################ ANGULAR

def w_z(theta, t):
    """
    Usage: angular velocity

    Variables: 
        theta = angle
        t = time"""
    return theta/t

def a_z(w, t):
    """
    Usage: angular acceleration

    Variables: 
        w_z = angular velocity
        t = time"""
    return w/t

def KinematicsRotationalEq(find, printEq=False, warning=True, **kwargs):
    """variables: 
                theta, theta0 = angle, initial angle
                w_z, w0_z = angular velocity, initial angular velocity
                a_z = angular acceleration (constant only in this case)
                t = time

        Usage: Energy Applied on an object"""
    if warning:
        print("Note: KineticsAngularEq() is valid for constant angular acceleration (a_z) only")
    eq = list()
    eq.append("Eq(theta, theta0 + w0_z*t + 0.5*a_z*t**2)")
    eq.append("Eq(theta - theta0, 0.5*(w_z + w0_z)*t)")
    eq.append("Eq(w_z, w0_z + a_z*t)")
    eq.append("Eq(w_z**2, w0_z**2 + 2*a_z*(theta - theta0))")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

################################ MOMENT OF INERTIA 
################################ KINETIC ENERGY

def momentOfInertia(m_r_touple_list):
    """
    Sample Input [(mass1, radius1), ..., (mass2, radius2)]
    
    Variables: 
        mass = mass
        radius = perpendicular distance from axis of rotation
    
    Usage: the higher the moment of inertia, the more 
    difficult it is to change the state of the body's rotation"""
    I = sum([m*r**2 for m,r in m_r_touple_list])
    return I

def kineticEnergy(I, w):
    """
    variables: 
        K = kinetic energy
        I = moment of inertia
        w = magnitude of angular velocity (w_z)
    """
    K = 0.5*I*w**2
    return K

def torque(find, printEq=False, warning=True, **kwargs):
    """
    variables:
        tau, tau_z = torque, torque along axis z
        F = force
        d = distance
        I = moment of inertia
        dw = change in angular speed
        dL = change in angular momentum
        dt = change in time
        dtheta = change in angle 
    """
    eq = list()
    eq.append("Eq(tau, F*d)")
    eq.append("Eq(tau, I*a_z)")
    eq.append("Eq(W,tau*dtheta)")
    eq.append("Eq(W,.5*I*dw**2)")
    eq.append("Eq(P,tau_z*w_z)")
    eq.append("Eq(dL/dt)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def angularMomentum(find, printEq=False, warning=True, **kwargs):
    eq = list()
    eq.append("Eq(L, I*w)")
    eq.append("Eq(tau, I*a_z)")
    eq.append("Eq(W,tau*dtheta)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)



import grtoolkit.Circuits
import grtoolkit.Decorators
import grtoolkit.Economics
import grtoolkit.File
import grtoolkit.Format
import grtoolkit.Hardcode
import grtoolkit.Heat
import grtoolkit.Math
import grtoolkit.Mechanics
import grtoolkit.MSAccess
import grtoolkit.PDF
import grtoolkit.Python
import grtoolkit.Quotes
import grtoolkit.Storage
import grtoolkit.TTS
import grtoolkit.Windows

#### EASY ACCESSS FUNCTIONS

import os, sys
def cwd():
    return os.path.dirname(sys.argv[0])


import math, scipy.constants
from grtoolkit.Math import solveEqs
from grtoolkit.Python import dictionary_add_modify

def projectileMotionEq(find=False, angleUnitRadians=True, printEq=False, **kwargs):
    """variables: 
            x=position-along-x, y=position-along-y,
            alpha0=original trajectory angle,
            t=time, g=standard acceleration of gravity
            vx=velocity-along-x, vy=velocity-along-y
            
        Note:
            Angle solution in radians"""

    if not(angleUnitRadians):
        dictionary_add_modify(kwargs,alpha0=radians(kwargs["alpha0"]))
    if not("g" in kwargs):
        dictionary_add_modify(kwargs,g=scipy.constants.g)

    posx="v0*cos(alpha0)*t"
    posy="v0*sin(alpha0)*t - 0.5*g*t**2"
    eq = list()
    eq.append(f"Eq(x,{posx})")
    eq.append(f"Eq(y,{posy})")
    eq.append(f"Eq(vx,diff({posx},t))")
    eq.append(f"Eq(vy,diff({posy},t))")
    eq.append(f"Eq(ax,diff({posx},t,t))")
    eq.append(f"Eq(ay,diff({posy},t,t))")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def projectileMotion2D(v0, alpha0, t, angleUnitRadians=True):
    """Quick version of projectile motion
    returns [x,y,vx,vy]"""
    g = scipy.constants.g
    alpha0 = radians(alpha0) if not(angleUnitRadians) else alpha0
    x=(v0*math.cos(alpha0))*t
    y=(v0*math.sin(alpha0))*t - 0.5*g*t**2
    vx=v0*math.cos(alpha0)
    vy=v0*math.sin(alpha0)-g*t
    return x, y, vx, vy


# python = "C:\Users\gabrielr\AppData\Local\Programs\Python\Python38-32\python.exe"
# pip = "C:\Users\gabrielr\AppData\Local\Programs\Python\Python38-32\python.exe\Scripts\pip.exe"

from pptx import Presentation
import sys, copy, six
from os.path import dirname
from grtoolkit.Storage import File

def get_blank_slide_layout(pres):
    layout_items_count = [len(layout.placeholders) for layout in pres.slide_layouts]
    min_items = min(layout_items_count)
    blank_layout_id = layout_items_count.index(min_items)
    return pres.slide_layouts[blank_layout_id]

def duplicate_slide(pres, index, no_of_copies=1):
    """Duplicate the slide with the given index in pres.
        Adds slide to the end of the presentation"""

    source = pres.slides[index]
    for _ in range(0,no_of_copies):

        blank_slide_layout = get_blank_slide_layout(pres)
        dest = pres.slides.add_slide(blank_slide_layout)

        for shp in source.shapes:
            el = shp.element
            newel = copy.deepcopy(el)
            dest.shapes._spTree.insert_element_before(newel, 'p:extLst')

        for _, value in six.iteritems(source.part.rels):
            # Make sure we don't copy a notesSlide relation as that won't exist
            if not "notesSlide" in value.reltype:
                dest.part.rels.add_relationship(value.reltype, value._target, value.rId)

    return dest


# CONNECT LIBRARIES
import grtoolkit.Mechanics.CircularMotion
import grtoolkit.Mechanics.Collisions
import grtoolkit.Mechanics.Friction
import grtoolkit.Mechanics.Kinematics
import grtoolkit.Mechanics.ProjectileMotion
import grtoolkit.Mechanics.Pulleys

# EASY ACCESS
import math, scipy.constants
from grtoolkit.Math import algebraSolve, solveEqs, magnitude
     
def forceEq(find, printEq=False, **kwargs):
    """variables: 
                W=Work  F=force  p=momentum  t=time
                x, x0, x1 = distance, start, finish
        Usage: Energy Applied on an object"""
    eq = list()
    eq.append("Eq(F,m*a)")
    eq.append("Eq(F,k*x)")
    eq.append("Eq(F,p/t)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def momentumEq(find, printEq=False, **kwargs):
    """variables: 
                m=mass  v=velocity
        Usage: Energy Applied on an object
        Sample Units: Kg*m/s"""
    eq = list()
    eq.append("Eq(p,m*v)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def impulseEq(find, printEq=False, **kwargs):
    """variables: 
                J=impulse  
                p1,p2=initial,final momentum
                F=sum of all forces
                t,t0,t1 = time initial,final
        Usage: The change in momentum of a particle during a time interval
        Sample Units: Kg*m/s"""
    eq = list()
    eq.append("Eq(J,p2-p1)")
    eq.append("Eq(J,integrate(F,(t,t0,t1)))")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def weight(m,g=scipy.constants.g):
    """variables: m=mass  
                  g=gravitational acceleration constant (9.81 m/s**2)"""
    return m*g

def kineticEnergy(mass,velocity):
    """1 J = 1 N*m = 1 Kg*m**2/s**2
    Usage: Energy moving an object"""
    K = 0.5*mass*velocity**2
    return K

def elasticPotentialEnergy(k,x):
    """1 J = 1 N*m = 1 Kg*m**2/s**2
    Variables: k=spring constant   x=distance compressed
    Usage: Energy stored by springs"""
    U = 0.5*k*x**2
    return U   

def gravitationalPotentialEnergy(mass, gravity, y):
    """1 J = 1 N*m = 1 Kg*m**2/s**2
    Variables: m=mass   g=gravity constant   y=height
    Usage: Energy stored by springs"""
    U = mass*gravity*y
    return U       

def workEq(find, printEq=False, **kwargs):
    """variables: 
                W=Work  F=force
                x, x0, x1 = distance, start, finish
        Usage: Energy Applied on an object"""
    eq = list()
    eq.append("Eq(W, F*x)")
    eq.append("Eq(W,integrate(F,(x,x0,x1)))")
    eq.append("Eq(W, P*t)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def powerEq(find, printEq=False, **kwargs):
    """variables: 
                P=power  W=Work  t=time
                f=force  v=velocity
        Base Unit = Watts"""
    eq = list()
    eq.append("Eq(P, W/t)")
    eq.append("Eq(P, F*v)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def centerOfMass(*args):
    """input: [m, x, y, z, r]
       variables: m=mass, 
                  x=position-along-x, 
                  y=position-along-y, 
                  z=position-along-z, 
                  r=position-along-radius"""
    mi=0; xi=1; yi=2; zi=3; ri=4
    m_total = sum([item[mi] for item in args])
    x = sum([item[mi]*item[xi] for item in args])/m_total
    y = sum([item[mi]*item[yi] for item in args])/m_total
    z = sum([item[mi]*item[zi] for item in args])/m_total
    r = sum([item[mi]*item[ri] for item in args])/m_total
    return x,y,z,r

def centerOfMassMotion(*args):
    """input: [m, vx, vy, vz, vr]
       variables: m=mass, 
                  vx=velocity-along-x, 
                  vy=velocity-along-y, 
                  vz=velocity-along-z, 
                  vr=velocity-along-radius"""
    mi=0; vxi=1; vyi=2; vzi=3; vri=4
    m_total = sum([item[mi] for item in args])
    vx = sum([item[mi]*item[vxi] for item in args])/m_total
    vy = sum([item[mi]*item[vyi] for item in args])/m_total
    vz = sum([item[mi]*item[vzi] for item in args])/m_total
    vr = sum([item[mi]*item[vri] for item in args])/m_total
    return vx,vy,vz,vr

def stress(f_normal,A):
    """variables:
            sigma=stress
            f_normal=force
            a=area"""
    sigma = f_normal/A
    return sigma

def shearStress(f_parallel,A):
    """variables:
            sigma_shear=shear stress (parallel)
            f=force
            a=area"""
    sigma_shear = f_parallel/A
    return sigma_shear
    
def strain(dl,l0):
    """variables:
            epsilon=strain
            dl=change in length
            l0=initial length"""
    epsilon = dl/l0
    return epsilon

def YoungsModulusEq(find="Y", printEq=False, **kwargs):
    """variables: 
            Y=young's modulus of elasticity
            sigma=stress
            epsilon=strain
    """
    eq = list()
    eq.append("Eq(Y, sigma/epsilon)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def shearModulus(find="S", printEq=False, **kwargs):
    """variables: 
            S = shear modulus
            sigma_shear = shear stress
            dx = along shear force (parallel to surface)
            h = height of subject that is subject to shear forces
    """
    eq = list()
    eq.append("Eq(S, sigma_shear/(dx/h))")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

if __name__ == "__main__":
    pass


import requests
from bs4 import BeautifulSoup, NavigableString
from grtoolkit.Storage import loadPickle, savePickle

def goodreads_scraper(author, page_num=None):
    ### SOURCE: https://soniajoseph.github.io/goodreads-scraper/
    ### If pagenum=None, then it will return all results.

	# old_author = author
	author = author.replace(" ", "+")
	all_quotes = []

	# if page number not specified, get true page number
	if page_num is None:
		try:
			page = requests.get("https://www.goodreads.com/quotes/search?commit=Search&page=1" + "&q=" + author + "&utf8=%E2%9C%93")
			soup = BeautifulSoup(page.text, 'html.parser')
			pages = soup.find(class_="smallText").text
			a = pages.find("of ")
			page_num = pages[a+3:]
			page_num = page_num.replace(",", "").replace("\n", "")
			page_num = int(page_num)
			print("looking through", page_num, "pages")
		except:
			page_num = 1

	# for each page
	for i in range(1, page_num+1, 1):
		try:
			page = requests.get("https://www.goodreads.com/quotes/search?commit=Search&page=" + str(i) + "&q=" + author + "&utf8=%E2%9C%93")
			soup = BeautifulSoup(page.text, 'html.parser')
			print("scraping page", i)
		except:
			print("could not connect to goodreads")
			break
			
		try:
			quote = soup.find(class_="leftContainer")
			quote_list = quote.find_all(class_="quoteDetails")
		except:
			pass

		# get data for each quote
		for quote in quote_list:
			meta_data = []
			# Get quote's text
			try:
				outer = quote.find(class_="quoteText")
				inner_text = [element for element in outer if isinstance(element, NavigableString)]
				inner_text = [x.replace("\n", "") for x in inner_text]
				final_quote = "\n".join(inner_text[:-4])
				meta_data.append(final_quote.strip())
			except:
				pass 

			# Get quote's author
			try:
				author = quote.find(class_="authorOrTitle").text
				author = author.replace(",", "")
				# author = author.replace("\n", "")
				meta_data.append(author.strip())
				# print(author)
			except:
				meta_data.append(None)

			# Get quote's book title
			try: 
				title = quote.find(class_="authorOrTitle")
				title = title.nextSibling.nextSibling.text
				# title = title.replace("\n", "")
				meta_data.append(title.strip())
				# print(title)
			except:
				meta_data.append(None)

			# Get quote's tags
			try:
				tags = quote.find(class_="greyText smallText left").text
				tags = [x.strip() for x in tags.split(',')]
				tags = tags[1:]
				meta_data.append(tags)
				# print(tags)
			except:
				meta_data.append(None)

			# Get number of likes
			try:
				likes = quote.find(class_="right").text
				likes = likes.replace("likes", "")
				likes = int(likes)
				meta_data.append(likes)
				# print(likes)
			except:
				meta_data.append(None)
			all_quotes.append(meta_data)

		# for text, author, title, tags, likes in all_quotes:
		# 	print(text)
		# 	print(author)
		# 	print(title)
		# 	print(tags)
		# 	print(likes)
		# 	print()
	return all_quotes

def generateDB(author, page_num=None, saveFile=None):
	db = goodreads_scraper(author, page_num)
	if saveFile != None:
		try:
			savePickle(saveFile,db)
		except:
			print("Error saving pickle...")
	return db

def loadDB(pickleFile):
	return loadPickle(pickleFile)

def bookTitles(db):
	titles = []
	for quote in db:
		if quote[2] not in titles:
			titles.append(quote[2])
	return titles

def filterQuotesByBook(db, book, min_likes=0):
    by_book = list()
    for quote in db:
        if quote[2] == book:
            if quote[4] >= min_likes:
                by_book.append(quote[0])
    return by_book

def export(quotes, saveFile, enc):
	from grtoolkit.Storage import File
	save = File(saveFile, enc=enc)

	for i in range(0,len(quotes)):
		if i == 1:
			save.write(quotes[i][1:-1])
			save.append("\n")
		else:
			save.append(quotes[i][1:-1],newline=False)
			save.append("\n")




if __name__ == "__main__":
    generateDB("jk rowling", 1)




#Code works; not optimized for grtoolkit library
import subprocess, os

def text2speech(text, audio_dest_path, play=False, 
                balcon_voice="ScanSoft Daniel_Full_22kHz", 
                balcon_location = r"C:\Users\Gabriel\Downloads\balcon\balcon.exe"):
    """Text to speech. Outputs .wav files"""
    command = f'{balcon_location} -t "{text}" -n {balcon_voice} -w "{audio_dest_path}"' 
    process = subprocess.call(command, shell=True)
    if play:
        os.startfile(audio_dest_path)

if __name__ == "__main__":
    text = """Hello Gabriel, it is a nice day today!"""
    dest = r"C:\Users\Gabriel\Git Repositories\YTAutomation\VoiceSynth\life.wav"
    text2speech(text, dest, play=True)



import functools, time


def try_pass(func):
    """Try/except/pass runtime of decorated function"""

    @functools.wraps(func)
    def wrapper_try_func(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except:
            pass

    return wrapper_try_func


def timer(func):
    """Print runtime of decorated function"""

    @functools.wraps(func)
    def wrapper_timer(*args, **kwargs):
        start_time = time.perf_counter()
        value = func(*args, **kwargs)
        end_time = time.perf_counter()
        run_time = end_time - start_time
        print(f"Finished {func.__name__!r} in {run_time:.4f} secs")
        return value

    return wrapper_timer


def debug(func):
    """Print the function signature and return value, useful for when a program is run multiple times.
    Example: Factorials"""

    @functools.wraps(func)
    def wrapper_debug(*args, **kwargs):
        # Do something before
        args_repr = [repr(a) for a in args]
        kwargs_repr = [f"{k}={v!r}" for k, v in kwargs.items()]
        signature = ", ".join(args_repr + kwargs_repr)
        print(f"Calling {func.__name__}({signature})")
        value = func(*args, **kwargs)
        # Do something after
        print(f"{func.__name__!r} returned {value!r}")
        return value

    return wrapper_debug


def slow_down(func, sec=1):
    """Sleep 1 second before calling function"""

    @functools.wraps(func)
    def wrapper_slow_down(*args, **kwargs):
        time.sleep(sec)
        return func(*args, **kwargs)

    return wrapper_slow_down


class wait(object):
    """Decorator to wait a specified number of seconds.
    Usage: @wait(seconds)"""

    # ref: https://python-3-patterns-idioms-test.readthedocs.io/en/latest/PythonDecorators.html
    def __init__(self, sec):
        self.sec = sec

    def __call__(self, func):
        @functools.wraps(func)
        def wrapper_wait(*args, **kwargs):
            time.sleep(self.sec)
            return func(*args, **kwargs)

        return wrapper_wait

if __name__ == "__main__":
    pass


from grtoolkit.Math import solveEqs

#UNTESTED

def Value(find, printEq=False, **kwargs):
    """
    variables: 
        Fs=simple future value,
        F=compound future value, 
        P=present value, 
        A=annuity (repeated payment)
        N=number of periods (ex. if 15yrs semiannually, N=30), 
        i_simple=simple interest,
        G=gradient, 
        i_growth=growth adjusted interest
        g=rate of growth
    """
    eq = list()
    eq.append("Eq(Fs,P*i_simple*N")
    eq.append("Eq(F,P*(1+i_simple)**N)")
    eq.append("Eq(A,F/N)") if kwargs["i_simple"] == 0 else eq.append("Eq(A,F*(i_simple/((1+i_simple)**N-1)))")
    eq.append("Eq(A,P*(i_simple*(1+i_simple)**N/((1+i_simple)**N-1)))")
    eq.append("Eq(A,G*1/i_simple-N/((1+i_simple)**N-1)")
    eq.append("Eq(P,A*((1+i_growth)**N-1)/(i_growth*(1+i_growth)**N)*(1/(1+g)))")   
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def i_simple(r,m):
    """
    variables: 
        r=nominal interest rate
        m=number of periods in a year
            Ex. annually = 1
                semianually = 2
                quaterly = 4
    """
    return r/m

def i_growth(i_simple,g):
    """
    variables: 
        g=rate of growth
    """
    return (1+i_simple)/(1+g)-1

def i_eff(printEq=False, **kwargs):
    """
    variables: 
        i_eff=effective interest
        r=
        m=

    """
    find="i_eff"
    eq = list()
    eq.append("Eq(i_eff,((1+r/m)**m)-1)")
    eq.append("Eq(i_eff,((1+i_simple)**m)-1)")
    eq.append("Eq(i_eff,exp(r)-1)")             #continuous
    return solveEqs(eq, find, printEq=printEq, **kwargs)





def collisionsRefresh():
    text = """
    3 different kinds of collisions. In all situations conservation of momentum is conserved.

    Perfectly elastic: All kinetic energy is conserved. Two colliding bodies perfectly exchange 
                       velocities.

    Inelastic: Some kinetic energy is conserved while some is lost to heat or sound.

    Completely inelastic: Colliding bodies have share a common final velocity with both mass 
                          moving in the same direction.
                          Example: two polar opposite magnets of the same mass collide and travel 
                                   in the direction of the net velocity at half the speed due to 
                                   their combined mass.
    """
    print(text)


import grtoolkit as gr 
import os, shutil

parentFolder = os.getcwd()

print("Generate Quotes from Goodreads website.\n")
author = input("Author to search:\n")
book = input("Book name as per Goodreads:\n")
pickleFile = f"{author}.pickle"
saveFile = f"Quotes.txt"

# IF PICKLE FILE EXISTS THEN LOAD, OTHERWISE GENERATE FROM GOODREADS WEBSITE
if os.path.isfile(pickleFile):
    db = gr.Quotes.loadDB(pickleFile)
else:
    db = gr.Quotes.generateDB(author, saveFile=pickleFile)

# FILTER DATABASE BY 
qts = gr.Quotes.filterQuotesByBook(db,book)
# EXPORT QUOTES TO NOTEPAD FILE
gr.Quotes.export(qts,f"{parentFolder}\\{saveFile}",enc=1)

# COPY QUOTES TEMPLATE FILE
shutil.copy2(os.environ['QUOTE_PPT_TEMPLATE'],f"{parentFolder}\\QUOTE_PPT_TEMPLATE.pptm")





import os, PyPDF2
from grtoolkit.File import filesInFolder


def join_PDFs(*args):
    """Join all PDFs at folder location or in list of absolute file locations"""

    if os.path.exists(args[0]):
        PDFs = filesInFolder(args[0], ".pdf")
    elif isinstance(args[0], list):
        PDFs = args[0]
    else:
        return "Folder location or file list not properly input."

    writer = PyPDF2.PdfFileWriter()  # NEW FILE
    for PDF in PDFs:
        try:
            pdfFile = open(args[0] + "\\" + PDF, "rb")
            reader = PyPDF2.PdfFileReader(pdfFile)
            for pageNum in range(reader.numPages):
                page = reader.getPage(pageNum)
                writer.addPage(page)
        except:
            pass
    outputFile = open(args[0] + "\\" + "JOINED_PDF.pdf", "wb")
    writer.write(outputFile)
    outputFile.close()
    pdfFile.close()



import math

def minPulleys(load_weight, pull_weight, Safety_factor, exact=False):
### Pull_weight = load_weight/(2**(n-1)) where load_weight is the heavier load.
    # You can use any base for the exp or log functions. The more used are 2, e and 10.
    # https://www.researchgate.net/post/Why_log_with_base_e_is_frequently_used_as_compared_to_base_10
    if exact:
        return math.log(load_weight/pull_weight)/math.log(2)
    return math.ceil(math.log(load_weight/pull_weight)/math.log(2))


import math, numpy as np
from sympy import *

def magnitude(*args):
    square_sum = 0
    for arg in args:
        square_sum += arg**2
    return math.sqrt(square_sum)

def roundSig(x, sig=5):
    if x == 0:
        return 0
    return round(x, sig - int(math.floor(math.log10(abs(x)))) - 1)

def delta(p1,p2):

    return abs(p2-p1)

def softmax(L):
    '''Function that takes as input a list of numbers, and returns the list of values given by the softmax function.
    
    Example:
    Input [1,2,3]
    Output [e**1/(e**1 + e**2 + e**3),
            e**2/(e**1 + e**2 + e**3)
            e**3/(e**1 + e**2 + e**3)]'''
    return [np.exp(x)/sum(np.exp(L)) for x in L]

def sigmoid(x):
    return 1/(1+np.exp(-x))

def cross_entropy(Y, P):
    '''Sample inputs: Y=[1,1,0] and P=[0.8,0.7,0.1] Equal 0.69
    Where,
    Y is list of whether events are on or off
    P is list of probabilities if events are on
    
    Formula is sum of all cases of entropy if event is active plus entropy if event is inactive
    
    Events with high probabilities have low cross entopy
    Events with low proabilities have high cross entropy

    OR

    High cross entropy means low probability
    Low cross entropy means high probability
    '''
   
    Y = np.float_(Y)
    P = np.float_(P)
    return -np.sum(Y * np.log(P) + (1 - Y) * np.log(1 - P))

def algebraSolve(expr, solve_for,**kwargs):
    """Solves algebraic string equation. 
    format: algebraSolve("x**2 + 3*x - 1/2 + y", "x", y=24)
    **"""
    expr=sympify(expr)
    #GENERATE VARIABLE SYMBOLS FROM EXPRESSION
    for sym in expr.free_symbols: 
        exec(f"{sym.name}=symbols('{sym.name}')")
        #MAKE solve_for STRING EQUAL REQUIRED EXPRESSION VARIABLE SYMBOL
        if solve_for == sym.name:    
            solve_for = sym
        
        # SUBSTITUTE KARGs FOR EXPRESSION SYMBOLS
        for k, v in kwargs.items():
            if k == sym.name:
                expr=expr.subs(k,v)
    return solve(expr, solve_for)

def solveEqs(eq, find, printEq=False, **kwargs):
    #CREATE SET OF ARGUMENTS SUPPLIED TO FUNCTION
    availSet = {find}
    for k in kwargs.keys():
        availSet.add(k)
    eq = [sympify(expr) for expr in eq]
    freesym = [expr.free_symbols for expr in eq]
    freevar = []
    solution = list()
    #CONVERT LIST OF SETS OF SYMBOLS INTO LIST OF SETS OF STR
    for freeset in freesym:
        tempset = set()
        for sym in freeset:
            tempset.add(sym.name)
        freevar.append(tempset)
    unknowns = [freeset-availSet for freeset in freevar]            # Eliminate variables requested in function arguments: find and knowns
    unknowns_count = [len(unknownset) for unknownset in unknowns]   # How many unknowns are left in each equation?
    find_avail = [find in freevarset for freevarset in freevar]     # Is the variable in question available in this equation?
    index = list(range(len(unknowns_count)))
    zipper = list(zip(index, unknowns_count, find_avail))

    for zippedList in zipper:
        solution.append(algebraSolve(eq[zippedList[0]],find, **kwargs))

    printEquationsSolution(eq, solution) #print Equations and Solutions to console
    return solution

def printEquations(eq):
    # print("Equations:\n")
    for equation in eq:
        print(equation)

def printEquationsSolution(eq, solution, printEq=True):
    #if printEq:
    print("\n")
    print("Equations"), printEquations(eq), print("\n")
    print("Solutions"), printEquations(solution), print("\n")

def solveEqSimultaneously():
    # Sympy Review https://docs.sympy.org/latest/modules/solvers/solvers.html
# Solve the following system:

#    x + 4 y ==  2
# -2 x +   y == 14

# system = Matrix(( (1, 4, 2), (-2, 1, 14)))

# solve_linear_system(system, x, y)
# {x: -6, y: 2}
    pass


from grtoolkit.Math import solveEqs

def kinematicsEq(find, printEq=False, **kwargs):
    """variables: 
                d=distance, d0=initial distance, 
                v=velocity, v0=initial velocity, 
                a=acceleration, 
                t=time"""
    eq = list()
    eq.append("Eq(d, v*t)")
    eq.append("Eq(v, v0 + a*t)")                #constant x-acceleration only
    eq.append("Eq(d, d0 + v0*t + 0.5*a*t**2)")  #constant x-acceleration only
    eq.append("Eq(d, d0 + v*t - 0.5*a*t**2)")
    eq.append("Eq(v**2, v0**2 + 2*a*d)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)


# PV = Planned value
# EV = Earned value
# AC = Actual Cost
# BAC = Budget at completion
# EAC = Estimate at completion
# ETC = Estimate to complete
# VAC = Variance at completion

#Cost Variance
def CostVariance(EV,AC):
    return EV-AC
def ScheduleVariance(EV,PV):
    return 
def CostPerformanceIndex(EV,AC):
    return EV/AC
def SchedulePerformanceIndex(EV,PV):
    return EV/PV

#use *args
# def EstimateAtCompletion(**kwargs):
def EstimateAtCompletion1(AC, BottomUpEAC):
    # Method1, used if original estimate is fundamentally flawed
    return AC + BottomUpEAC
def EstimateAtCompletion2(BAC,CPI=None, EV=None, AC=None):
    # Method2, no variances from BAC or expected continued rate of spending
    # if EV and AC:
    #     return BAC/CPI
    # else:
    #     return BAC/CPI(EV,AC)
    if EV: print(EV)

#     return BAC/CPI(EV,AC)
#     return BAC/CPI
#     # Method3, used when current variances are though to be atypical of future trends
#     return AC + (BAC-EV)
#     # Method4, used when current variances are thought to be typical of future trends and project schedule constraints will influence the completion of the remaining effort
#     return AC + ((BAC - EV)/
#                 (CPI * SPI))
    # KeyCheck(kwargs,"boo")
    # if ('boo' in kwargs) and ('sugar' in kwargs):
    # if KeyCheck(kwargs,"boo"):
    #     print("scared you")
    # else:
    #     print("hoo")

def ToCompletePerformanceIndex(BAC, EV, AC):
    return (BAC-EV)/(BAC-AC) #divides the value of work remaining by the money remaining to do it
def EstimateToComplete(EAC, AC):
    return EAC - AC #or reestimate
def VarianceAtCompletion(BAC,EAC):
    return BAC - EAC


# KeyCheck = lambda kwargs,*args: args in kwargs
def KeyCheck(kwargs,*args):
    for key in kwargs:
        if not(key in args):
            return False
    return True



#Multiple names for functions
CV = CostVariance
SV = ScheduleVariance
CPI = CostPerformanceIndex
SPI = SchedulePerformanceIndex
EAC1 = EstimateAtCompletion1
EAC2 = EstimateAtCompletion2
TCPI = ToCompletePerformanceIndex
ETC = EstimateToComplete
VAC = VarianceAtCompletion

print(EAC2(26000,EV=65000,AC=75000))


# An op amp is an active circuit element designed to perform mathematical operations of addition, subtraction, multiplication, division, differentiation, and integration.

from grtoolkit.Math import solveEqs

def referenceMaterial():
    #Open reference material
    pass

def invertingAmplifier(find="v_out", printEq=False, **kwargs):
    """variables: 
                v_out, v_in = voltage out/in
                R2, R1 = resistors (view reference image)
    |image2|

    .. image:: OpAmp_Summary.PNG

    .. |image2| image:: OpAmp_Summary.PNG

    grtoolkit\Circuits\OpAmps\OpAmp_Summary.PNG

    """
    eq = list()
    eq.append("Eq(v_out,-R2/R1*v_in)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def nonInvertingAmplifier(find="v_out", printEq=False, **kwargs):
    """variables: 
                v_out, v_in = voltage out/in
                R2, R1 = resistors (view reference image)
                """
    eq = list()
    eq.append("Eq(v_out,(1+R2/R1)*v_in)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def voltageFollower(find="v_out", printEq=False, **kwargs):
    """variables: 
                v_out, v_in = voltage out/in (view reference image)
                """
    eq = list()
    eq.append("Eq(v_out,v_in)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def summer(find="v_out", printEq=False, **kwargs):
    """variables: 
                v_out, v1, v2, v3 = voltage out/in 
                Rf, R3, R2, R1 = resistors (view reference image)
                """
    eq = list()
    eq.append("Eq(v_out,-(Rf/R1*v1 + Rf/R2*v2 + Rf/R3*v3))")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def differenceAmplifier(find="v_out", printEq=False, **kwargs):
    """variables: 
                v_out, v1, v2, v3 = voltage out/in 
                R2, R1 = resistors (view reference image)
                """
    eq = list()
    eq.append("Eq(v_out,R2/R1*(v2-v1)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)


import pyodbc

def SQLQuery(dbFile, SQL, fetch="all", commit=False):
    """commit False = SELECT, commit True = CREATE, UPDATE, DELETE"""
    databaseFile  = dbFile
    connectionString = "Driver={Microsoft Access Driver (*.mdb, *.accdb)};Dbq=%s" % databaseFile
    dbConnection   = pyodbc.connect(connectionString)
    cursor = dbConnection.cursor()
    cursor.execute(SQL)
    if commit:
        cursor.commit()
        cursor.close()
        dbConnection.close()
    else:
        if fetch == "all":
            rows = cursor.fetchall() 
        elif fetch == "one":
            rows = cursor.fetchone() 
        elif fetch == "val":
            rows = cursor.fetchval()
        else:
            assert fetch == "all" or fetch == "one" or fetch == "val", print("Incorrect fetch mode.")
        cursor.close()
        dbConnection.close()
        if rows:
            return rows

    #HOW TO CALL ON ROWS
    # for row in cursor:
    # print(row.user_id, row.user_name)

    # for row in rows:
    #     print(row.temp)


from grtoolkit.Math import solveEqs

def capacitor(find, printEq=False, **kwargs):
    """
    usage: capacitor is an open circuit to DC
           voltage cannot change abrumptly in a capacitor

    variables: 
                q=charge
                C=capacitance
                v, v0, v1=voltage initial, final
                epsilon = permitivity of the dielectric material between plates
                w=work
                """
    eq = list()
    eq.append("Eq(q,C*v)")    
    eq.append("Eq(C,epsilon*A*d)")
    eq.append("Eq(i,C*dv/dt)")
    eq.append("Eq(w,0.5*C*v**2)") 
    eq.append("Eq(w,integrate(C*v,(v,v0,v1)))")
    eq.append("Eq(w,q**2/(2*C))")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def capacitorsInSeries(c_list):
    sumOfInverse = sum([1/c for c in c_list])
    return 1/sumOfInverse

def capacitorsInParallel(c_list):
       return sum(c_list)

def inductor(find, printEq=False, **kwargs):
    """
    usage: 
        measured in Farads (F)
        inductor is an short circuit in DC
        current cannot change abrumptly in a capacitor

    variables: 
        v=voltage
        L=inductance
        di=change in current
        t, t0, t1, dt=time, initial time, future time, change in time
        i, i0, di = current, initial current, change in current
        w=work (J)
    """
    eq = list()
    eq.append("Eq(v,L*di/dt)")
    eq.append("Eq(i,1/L*integrate(v,(t,t0,t1))+i0)")
    eq.append("Eq(w,0.5*L*i**2)")
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def inductance(find="L", printEq=False, **kwargs):
    """
    usage: measured in henry (H)

    variables: 
                N=number of turns
                A=cross sectional area
                l=length
                mu=permeability of core"""
    eq = list()
    eq.append("Eq(L,N**2,mu*A/l)")    
    return solveEqs(eq, find, printEq=printEq, **kwargs)

def inductorsInSeries(l_list):
    return sum(l_list)

def inductorsInParallel(l_list):
    sumOfInverse = sum([1/l for l in l_list])
    return 1/sumOfInverse


import sys, re, os, math, functools, time

def replaceWords(textOrFile, dictionary):
    from grtoolkit.Storage import File
    '''Replace words in content by dictionary keys and values'''
    base_text = textOrFile
    if os.path.exists(textOrFile):
        base_text = File(textOrFile).read()
    for key, val in dictionary.items():
        try:
            base_text = base_text.replace(key,val)
        except:
            pass
    return base_text

def foldersInFolder(folder, query=""):  
    '''Returns list of files of specified file type'''
    folder_list = []
    for _, subfolders, _ in os.walk(folder):  # folders, subfolders, filenames
        for sub in subfolders:
            if query in sub:  # if file_search is not empty
                folder_list.append(sub)
    return folder_list


def filesInFolder(folder, fileType="*"):  
    '''Returns list of files of specified file type'''
    fileType = delDotPrefix(fileType)
    file_regex = re.compile(
        r".*\." + fileType, re.IGNORECASE
    )  # Regular Expression; dot star means find everything
    file_list = []
    for _, _, filenames in os.walk(folder):  # folders, subfolders, filenames
        for singlefile in filenames:
            file_search = file_regex.findall(singlefile)
            if file_search:  # if file_search is not empty
                file_list.append(singlefile)
    return file_list


def directoryLastValue(directory):
    '''Returns the last value in the directory'''
    return directory.rsplit("\\", 1)[-1]


def checkExtension(filePath, ext):
    '''CompareS file extension; returns True/False'''
    return filePath[filePath.rfind(".", 0) : len(filePath)] == correctDotPrefix(ext)


def correctExtension(filePath, ext):
    '''Add file extension if it is incorrect'''
    return (
        filePath if checkExtension(filePath, ext) else filePath + correctDotPrefix(ext)
    )

def correctDotPrefix(string):
    '''Add dot prefix to file extension if missing'''
    return string if string.find(".") == 0 else "." + string


def delDotPrefix(string):
    '''Delete dot prefix to file extension if present'''
    return string[1:] if string.find(".") == 0 else string

def ext(path, dot=True):
    '''Extracts extension from file name'''
    if dot:
        return path[path.rfind(".", 0) : len(path)]
    else:
        return path[path.rfind(".", 0) + 1 : len(path)]

def name(path):
    '''Extracts file name without extension'''
    return path[: path.rfind(".", 0)]

if __name__ == "__main__":
    pass